@@ -10,18 +10,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from keystonemiddleware import exceptions
 
-class InvalidToken(Exception):
+
+ConfigurationError = exceptions.ConfigurationError
+
+
+class InvalidToken(exceptions.KeystoneMiddlewareException):
     pass
 
 
-class ServiceError(Exception):
+class ServiceError(exceptions.KeystoneMiddlewareException):
     pass
 
 
-class ConfigurationError(Exception):
-    pass
-
-
-class RevocationListError(Exception):
+class RevocationListError(exceptions.KeystoneMiddlewareException):
     pass
