@@ -17,8 +17,8 @@
 Utilities for memcache encryption and integrity check.
 
 Data should be serialized before entering these functions. Encryption
-has a dependency on the pycrypto. If pycrypto is not available,
-CryptoUnavailableError will be raised.
+has a dependency on the cryptography module. If cryptography is not
+available, CryptoUnavailableError will be raised.
 
 This module will not be called unless signing or encryption is enabled
 in the config. It will always validate signatures, and will decrypt
@@ -33,16 +33,18 @@ import hashlib
 import hmac
 import math
 import os
-import six
-import sys
 
 from keystonemiddleware.i18n import _
 
-# make sure pycrypto is available
 try:
-    from Crypto.Cipher import AES
+    from cryptography.hazmat import backends as crypto_backends
+    from cryptography.hazmat.primitives import ciphers
+    from cryptography.hazmat.primitives.ciphers import algorithms
+    from cryptography.hazmat.primitives.ciphers import modes
+    from cryptography.hazmat.primitives import padding
 except ImportError:
-    AES = None
+    ciphers = None
+
 
 HASH_FUNCTION = hashlib.sha384
 DIGEST_LENGTH = HASH_FUNCTION().digest_size
@@ -73,37 +75,15 @@ class CryptoUnavailableError(Exception):
 
 
 def assert_crypto_availability(f):
-    """Ensure Crypto module is available."""
+    """Ensure cryptography module is available."""
     @functools.wraps(f)
     def wrapper(*args, **kwds):
-        if AES is None:
+        if ciphers is None:
             raise CryptoUnavailableError()
         return f(*args, **kwds)
     return wrapper
 
 
-if sys.version_info >= (3, 3):
-    constant_time_compare = hmac.compare_digest
-else:
-    def constant_time_compare(first, second):
-        """Return True if both string inputs are equal, otherwise False.
-
-        This function should take a constant amount of time regardless of
-        how many characters in the strings match.
-
-        """
-        if len(first) != len(second):
-            return False
-        result = 0
-        if six.PY3 and isinstance(first, bytes) and isinstance(second, bytes):
-            for x, y in zip(first, second):
-                result |= x ^ y
-        else:
-            for x, y in zip(first, second):
-                result |= ord(x) ^ ord(y)
-        return result == 0
-
-
 def derive_keys(token, secret, strategy):
     """Derive keys for MAC and ENCRYPTION from the user-provided secret.
 
@@ -117,6 +97,15 @@ def derive_keys(token, secret, strategy):
     This approach is faster than computing a separate hmac as the KDF
     for each desired key.
     """
+    if not isinstance(secret, bytes):
+        secret = secret.encode()
+
+    if not isinstance(token, bytes):
+        token = token.encode()
+
+    if not isinstance(strategy, bytes):
+        strategy = strategy.encode()
+
     digest = hmac.new(secret, token + strategy, HASH_FUNCTION).digest()
     return {'CACHE_KEY': digest[:DIGEST_SPLIT],
             'MAC': digest[DIGEST_SPLIT: 2 * DIGEST_SPLIT],
@@ -126,6 +115,12 @@ def derive_keys(token, secret, strategy):
 
 def sign_data(key, data):
     """Sign the data using the defined function and the derived key."""
+    if not isinstance(key, bytes):
+        key = key.encode()
+
+    if not isinstance(data, bytes):
+        data = data.encode()
+
     mac = hmac.new(key, data, HASH_FUNCTION).digest()
     return base64.b64encode(mac)
 
@@ -137,24 +132,39 @@ def encrypt_data(key, data):
     Padding is n bytes of the value n, where 1 <= n <= blocksize.
     """
     iv = os.urandom(16)
-    cipher = AES.new(key, AES.MODE_CBC, iv)
-    padding = 16 - len(data) % 16
-    return iv + cipher.encrypt(data + six.int2byte(padding) * padding)
+    cipher = ciphers.Cipher(
+        algorithms.AES(key),
+        modes.CBC(iv),
+        backend=crypto_backends.default_backend())
+
+    # AES algorithm uses block size of 16 bytes = 128 bits, defined in
+    # algorithms.AES.block_size. Previously, we manually padded this using
+    # bytes((padding,)) * padding.  Using ``cryptography``, we will
+    # analogously use hazmat.primitives.padding to pad it to
+    # the 128-bit block size.
+    padder = padding.PKCS7(algorithms.AES.block_size).padder()
+    padded_data = padder.update(data) + padder.finalize()
+    encryptor = cipher.encryptor()
+    return iv + encryptor.update(padded_data) + encryptor.finalize()
 
 
-@assert_crypto_availability
 def decrypt_data(key, data):
     """Decrypt the data with the given secret key."""
     iv = data[:16]
-    cipher = AES.new(key, AES.MODE_CBC, iv)
+    cipher = ciphers.Cipher(
+        algorithms.AES(key),
+        modes.CBC(iv),
+        backend=crypto_backends.default_backend())
     try:
-        result = cipher.decrypt(data[16:])
+        decryptor = cipher.decryptor()
+        result = decryptor.update(data[16:]) + decryptor.finalize()
     except Exception:
         raise DecryptError(_('Encrypted data appears to be corrupted.'))
 
     # Strip the last n padding bytes where n is the last value in
     # the plaintext
-    return result[:-1 * six.byte2int([result[-1]])]
+    unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()
+    return unpadder.update(result) + unpadder.finalize()
 
 
 def protect_data(keys, data):
@@ -192,7 +202,7 @@ def unprotect_data(keys, signed_data):
         signed_data[DIGEST_LENGTH_B64:])
 
     # Then verify that it matches the provided value
-    if not constant_time_compare(provided_mac, calculated_mac):
+    if not hmac.compare_digest(provided_mac, calculated_mac):
         raise InvalidMacError(_('Invalid MAC; data appears to be corrupted.'))
 
     data = base64.b64decode(signed_data[DIGEST_LENGTH_B64:])
